import java.io.*;
import java.util.*;

import dk.vajhoej.record.*;
import dk.vajhoej.isam.*;
import dk.vajhoej.isam.cache.*;
import dk.vajhoej.isam.local.*;
import dk.vajhoej.isam.remote.*;

public class Test1 {
    private static final int N = 10000;
    private static Random rng = new Random();
    public static void testRead(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Read 1-100:");
        for(int i = 1; i <= 100; i++) {
            Data1 o = isam.read(Data1.class, new Key<Integer>(0, i));
            System.out.println(o.getId() + " = |" + o.getVal() + "|");
        }
    }
    public static void testCreate(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Create 101:");
        Data1 o = isam.read(Data1.class, new Key<Integer>(0, 101));
        if(o != null) {
            System.out.println("unexpected: " + o.getId() + " = |" + o.getVal() + "|");
        } else {
            System.out.println("expected: not found");
        }
        Data1 o2 = new Data1();
        o2.setId(101);
        o2.setVal("Testing");
        isam.create(o2);
        o = isam.read(Data1.class, new Key<Integer>(0, 101));
        if(o != null) {
            System.out.println("expected: " + o.getId() + " = |" + o.getVal() + "|");
        } else {
            System.out.println("unexpected: not found");
        }
    }
    public static void testUpdate(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Update 101:");
        Data1 o = isam.read(Data1.class, new Key<Integer>(0, 101));
        o.setVal("Testing - changed");
        isam.update(o);
        Data1 o2 = isam.read(Data1.class, new Key<Integer>(0, 101));
        System.out.println(o2.getId() + " = |" + o2.getVal() + "|");
    }
    public static void testDelete(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Delete 101:");
        isam.delete(Data1.class, new Key<Integer>(0, 101));
        Data1 o = isam.read(Data1.class, new Key<Integer>(0, 101));
        if(o != null) {
            System.out.println("unexpected: " + o.getId() + " = |" + o.getVal() + "|");
        } else {
            System.out.println("expected: not found");
        }
    }
    public static void testXX(IsamResult<Data1> res) throws IsamException, RecordException {
        while(res.read()) {
            Data1 o = res.current();
            System.out.println(o.getId() + " = |" + o.getVal() + "|");
        }
        res.close();
    }
    public static void testGE(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Read >=90:");
        testXX(isam.readGE(Data1.class, new Key<Integer>(0, 90)));
    }
    public static void testGT(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Read >90:");
        testXX(isam.readGT(Data1.class, new Key<Integer>(0, 90)));
    }
    public static void testLE(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Read <=10:");
        testXX(isam.readLE(Data1.class, new Key<Integer>(0, 10)));
    }
    public static void testLT(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Read <10:");
        testXX(isam.readLT(Data1.class, new Key<Integer>(0, 10)));
    }
    public static void testPerf(IsamSource isam) throws IsamException, RecordException {
        System.out.println("Performance:");
        long t1 = System.currentTimeMillis();
        for(int i = 0; i < N; i++) {
            Data1 o = isam.read(Data1.class, new Key<Integer>(0, rng.nextInt(100)+1));
        }
        long t2 = System.currentTimeMillis();
        System.out.printf("%.1f reads per second\n", N * 1000.0 / (t2 - t1));
    }
    public static void test(IsamSource isam) throws IsamException, RecordException {
        testRead(isam);
        testCreate(isam);
        testUpdate(isam);
        testDelete(isam);
        testGE(isam);
        testGT(isam);
        testLE(isam);
        testLT(isam);
        testPerf(isam);
    }
    public static void main(String[] args) {
        try {
            System.out.println("**** simple struct ****");
            System.out.println("local");
            System.out.println("=====");
            IsamSource local = new LocalIsamSource("data1.isq", "dk.vajhoej.vms.rms.IndexSequential", false);
            test(local);
            System.out.println("cache");
            System.out.println("=====");
            IsamSource cache = new CacheIsamSource(local);
            test(cache);
            System.out.printf("cache hit rate: %.4f\n", ((CacheIsamSource)cache).getHitRate());
            cache.close();
            local.close();
            if(args.length > 0 && args[0].toLowerCase().equals("noremote")) System.exit(0);
            System.out.print("Start server and press return...");
            System.in.read();
            System.out.println("remote (object)");
            System.out.println("===============");
            IsamSource remote = new RemoteIsamSource("localhost", 10001);
            test(remote);
            remote.close();
        } catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}    
